###source("/usr1/webserver/cgibin/rnacon/progs/IPKnotGraph2-BPSEQ.R/IPKnotGraph2-BPSEQ.R")

library(Biostrings)
library(igraph)

## BPSEQ format:
## # tRNA|Arg1|ACG|Escherichia coli|cytosolic
#1 G 73
#2 C 72
#3 A 71
#4 U 70
#5 C 69
##
## IMPORTANT----->> 
## as the description starts with a #.. we have to replace # with a > to be able to read it in a fasta format or even for read.table()

address<-commandArgs(TRUE)
#address1<-"/usr1/webserver/cgidocs/tmp/rnacon"
#print(address)
Graph_output_address<-paste(address, "/graph" ,sep="")
#IP_Data_file<-"/Users/bharat/ncRNA/IPknotGraph/BPSEQ.format"
IP_Data_file<-list.files(paste(address, "/structure",sep=""), pattern = ".out$", full.names = TRUE)

##Fasta_Seq_file<-"/Users/bharat/ncRNA/IPknotGraph/sample-fasta"
Fasta_Seq_file<-list.files(paste(address, "/fasta",sep=""), full.names = TRUE)

for(IP in 1:length(IP_Data_file))
{
	IP_Data<-read.table(IP_Data_file[IP], header=FALSE, sep=" ", na.strings="NA", dec=",", strip.white=TRUE, fill=TRUE) 
	#dim(IP_Data)## 10790 3
	#no_lines<-length(IP_Data[,1])##10790 lines

	IP_RNA_name<-gsub(".out", "", gsub(paste(address, "/structure/",sep=""), "", IP_Data_file[IP]))

	## match names between IP_Data_files and Fasta files here
	
	Fasta_seq<-readFASTA(Fasta_Seq_file[grep(IP_RNA_name, Fasta_Seq_file)], strip.descs=FALSE)

	##Fasta_seq<-read.RNAStringSet(Fasta_Seq_file[grep(IP_RNA_name, Fasta_Seq_file)], format="fasta")

	## IMP---find the indexes where the RNA starts.. find ">"
	reg1<-regexpr(">", IP_Data$V1)
	positions_IP_Data<-which(reg1 == 1)

	Graph_properties_outfile1<-paste(Graph_output_address, "/",IP_RNA_name,"_graph.csv", sep="")

	cat(paste("## Articulation points, Average path length, Average node betweenness, Variance of node betweenness, Average edge betweenness, Variance of edge betweenness, Average co-citation coupling, Average bibliographic coupling, Average closeness centrality, Variance of closeness centrality, Average Burt's constraint, Variance of Burt's constraint, Average degree, Diameter, Girth, Average coreness, Variance of coreness, Maximum coreness, Graph density, Transitivity, \n"), file=Graph_properties_outfile1, append=TRUE)

	count_fasta_seq<-0
	## make this look faster by preallocating list sizes and doing list[i]<-a[i]
	for(l1 in positions_IP_Data)
	{
		count_fasta_seq<-count_fasta_seq+1
		desc_rna<-Fasta_seq[[count_fasta_seq]]$desc
		seq_len<-nchar(Fasta_seq[[count_fasta_seq]]$seq)

		position_rna<-as.vector(IP_Data$V1[(l1+1):(l1+seq_len)])
		seq_rna<-as.vector(IP_Data$V2[(l1+1):(l1+seq_len)])
		link_rna<-as.vector(IP_Data$V3[(l1+1):(l1+seq_len)])

		edgeMatrix<-matrix(as.integer(c(position_rna, link_rna)), nc=2)

		################
		### IMPORTANT igraph considers only present links in the matrix... therefore 0 would also be considered as a node
		### whereas IPKnot considers 0 as no link... have to delete all the rows that have second column as 0
		################		
		edgeMatrix<-edgeMatrix[edgeMatrix[,2] != 0,]

		ids_from<-1:(seq_len-1)
		ids_to<-(1+1):(seq_len)

		consecutive_edges <- matrix(c(ids_from, ids_to), nc=2)
		g<-graph(t(consecutive_edges), directed=FALSE)
		g<-add.edges(g, t(edgeMatrix))

		#########
		### GRAPH PROPERTIES
		#########

		## IMP----->>> Check GraPPLE outputs and see what properties they use... maybe avoid matrix??
		shortest.paths<-shortest.paths(g)
		average.path.length<-average.path.length(g)
		articulation.points<-articulation.points(g)
		betweenness<-betweenness(g)# node
		edge.betweenness<-edge.betweenness(g)
		cocitation<-cocitation(g)
		bibcoupling<-bibcoupling(g)
		closeness<-closeness(g)
		constraint<-constraint(g)
		degree<-degree(g)
		diameter<-diameter(g)
		girth<-girth(g)
		graph.coreness<-graph.coreness(g)
		
		#Coreness-->The coreness of a vertex is k if it belongs to the k-core but not to the (k+1)-core Biological interpretation: Every node in graphs that represent ncRNA secondary structure has a coreness of two except for those in the dangling ends which have a coreness of one. Thus, the coreness is a measure of the total length of the dangling ends ranging between 1 and 2, where an average coreness of 2 means no dangling ends.
		graph.density<-graph.density(g)
		transitivity<-transitivity(g)

		cat(paste(mean(articulation.points, na.rm = TRUE), ",",  average.path.length, ",", mean(betweenness, na.rm = TRUE), ",", var(betweenness, na.rm = TRUE), ",", mean(edge.betweenness, na.rm = TRUE), ",", var(edge.betweenness, na.rm = TRUE), ",", mean(apply(cocitation, 1, mean)), ",", mean(apply(bibcoupling, 1, mean)), ",", mean(closeness, na.rm = TRUE), ",", var(closeness, na.rm = TRUE), ",", mean(constraint, na.rm = TRUE), ",", var(constraint, na.rm = TRUE), ",", mean(degree, na.rm = TRUE), ",", diameter, ",", girth$girth, ",", mean(graph.coreness, na.rm = TRUE), ",", var(graph.coreness, na.rm = TRUE), ",", max(graph.coreness, na.rm = TRUE), ",", graph.density, ",", transitivity, "\n"), file=Graph_properties_outfile1, append=TRUE)

	}
}
